/* bstrap.c */

#include "nbp.h"

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* External variables
 */
extern int usepxe;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Global variables
 */
BOOTPLAYER discover;
BOOTPLAYER dhcpack;
BOOTPLAYER proxy;
BOOTPLAYER reply;

t_IP gateway;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* unsigned nbpentry(pxeptr, pxenvptr)
 *	C entry point for PXE bootstrap programs.
 *
 * passed:
 *	t_PXE far *pxeptr := Far pointer to !PXE structure
 *	t_PXENV far *pxenvptr := Far pointer to PXENV+ structure
 *
 * returns:
 *	16-bit status that will be returned to PXE boot ROM
 */
unsigned
nbpentry(unsigned long pxeptr, unsigned long pxenvptr)
{
	t_DHCP_ROUTER *router;
	UINT16 type;

	bputs("\nIntel X86PC Bootstrap, Beta-3 (build 003)\n");

	/*
	 * Verify PXE structures and initialize global PXE variables.
	 */

	if (!pxeinit(pxeptr, pxenvptr)) {
		bputs("\nPXE-E81: !PXE structure is invalid.\n");
		return PXENV_STATUS_FAILURE;
	}

	/*
	 * Get cached packets from PXE boot ROM
	 */

	if (!get_cached_packet(PXENV_PACKET_TYPE_DHCP_DISCOVER, &discover)) {
		bputs("\nCould not get cached DHCP Discover packet.\n");
		return PXENV_STATUS_FAILURE;
	}

	if (!get_cached_packet(PXENV_PACKET_TYPE_DHCP_ACK, &dhcpack)) {
		bputs("\nCould not get cached DHCP Ack packet.\n");
		return PXENV_STATUS_FAILURE;
	}

	if (!get_cached_packet(PXENV_PACKET_TYPE_BINL_REPLY, &proxy)) {
		bputs("\nCould not get cached Proxy packet.\n");
		return PXENV_STATUS_FAILURE;
	}

	/*
	 * Initialize default gateway for UDP and TFTP API calls.
	 */

	if (usepxe) {
		/*
		 * Default gateway is 0.0.0.0 for PXE 2.0 ROMs.
		 */

		gateway = LONGSWAP(0);
	} else {
		/*
		 * Get default gateway from DHCP Ack packet
		 */

		if (router = find_dhcp_tag(&dhcpack, DHCP_ROUTER, 0))
			gateway = *(t_IP *)router->ip;
		else
			gateway = *(t_IP *)dhcpack.gip;

		/*
		 * If there is no default gateway in the DHCP Ack packet,
		 * but there is one in the Proxy packet, use the one in
		 * Proxy packet.
		 */

		if (!gateway) {
			if (router = find_dhcp_tag(&proxy, DHCP_ROUTER, 0))
				gateway = *(t_IP *)router->ip;
			else
				gateway = *(t_IP *)proxy.gip;
		}
	}

	/*
	 * Display and process remote boot prompt and menu
	 */

	if (!(type = do_menu())) {
		/*
		 * A return value of zero means the remote boot has been
		 * canceled.  Return control to PXE boot ROM so it can
		 * clean up and return control to the BIOS so the next
		 * boot device can be tried.
		 */

		return PXENV_STATUS_SUCCESS;
	}

	/*
	 * Discover selected boot server type.  The first discovery
	 * layer is always zero.  Non-zero discovery layers will
	 * only do unicast discovery to the PXE boot server that is
	 * in the cached proxy packet.
	 */

	switch (do_discover(type, 0)) {
	case -2: /* error */
		/* 
		 * A discovery error occured.  Return to PXE boot ROM.
		 */

		return PXENV_STATUS_FAILURE;

	case -1: /* abort */
		/*
		 * Discovery aborted by user.  Return to PXE boot ROM.
		 */

		return PXENV_STATUS_SUCCESS;

	case 0: /* not found */
		/*
		 * Boot server not found.  Return to PXE boot ROM.
		 */

		return PXENV_STATUS_SUCCESS;
	}

	/*
	 * Write modified Discover and Proxy cached packets to PXE
	 * boot ROM data area.  This information is needed by the
	 * next layer downloaded.
	 */

	if (!set_cached_packet(PXENV_PACKET_TYPE_DHCP_DISCOVER, &discover)) {
		bputs("\nCould not set cached DHCP Discover packet.\n");
		return PXENV_STATUS_FAILURE;
	}

	if (!set_cached_packet(PXENV_PACKET_TYPE_BINL_REPLY, &reply)) {
		bputs("\nCould not set cached Proxy packet.\n");
		return PXENV_STATUS_FAILURE;
	}

	/*
	 * Download next layer.  If control returns, an error or abort
	 * occured.  Return control to PXE boot ROM.
	 */

	do_restart();

	return PXENV_STATUS_FAILURE;
}
