/* misc.h */

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* int _DS()
 *	Return DS register.
 */
extern int _DS();

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* int _SS()
 *	Return SS register.
 */
extern int _SS();

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* int fbms()
 *	Return free base memory size in Kbytes.
 */
extern int fbms();

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
int setfbms(int newfbms);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* int switch_stack()
 *	Switch between PXE option ROM CPU stack and NBP CPU stack.
 *	This must be done before running PXENV_RESTART_TFTP and
 *	PXENV_UNLOAD_STACK APIs.
 *	Do not use any stack (local) variables/pointers after
 *	switching to the PXE ROM CPU stack.
 */
extern int switch_stack();

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* int _fmemcpy(to, from, bytes)
 *	Copy up to 65535 bytes of memory from one seg:off pointer to
 *	another.  Call only works in real mode.  seg:off pointer must
 *	be created using MK_FP() macro.
 *
 * passed:
 *	void far *to := Copy to this seg:off pointer
 *	void far *from := Copy from this seg:off pointer
 *	int bytes := Number of bytes to copy
 *
 * returns:
 *	Number of bytes copied.
 */
extern int _fmemcpy(UINT32 to, UINT32 from, int bytes);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* UINT8 cksum(buf, bytes)
 *	Compute 8-bit checksum of buffer contents.
 *
 * passed:
 *	UINT8 *buf := Pointer to buffer
 *	int bytes := Number of bytes to checksum.
 *
 * returns:
 *	8-bit checksum of buffer
 */
extern UINT8 cksum(UINT8 *buf, int bytes);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* long tickcount()
 *	Return system tick count from BIOS data segment (0x40:0x6C).
 *
 * returns:
 *	Number of ticks since midnight.
 */
extern long tickcount();

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* swapattr(row, col, count)
 *	Swap display attributes on page zero of colour and monochrome
 *	text consoles.
 *
 * passed:
 *	int row := Row number.  Top of screen is zero.
 *	int col := Column number.  Left side of screen is zero.
 *	int count := Number of character positions to swap.
 */
extern int swapattr(int row, int col, int count);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* int getrow()
 *	Return the current cursor row number on page zero.
 *
 * returns:
 *	Row number.  Top of screen is zero.
 */
extern int getrow();

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* int getcol()
 *	Return the current cursor column number on page zero.
 *
 * returns:
 *	Column number.  Left side of screen is zero.
 */
extern int getcol();

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* int getrows()
 *	Return the number of rows on the screen.
 *
 * returns:
 *	Number of rows.
 */
extern int getrows();

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* int getcols()
 *	Return the number of columns on the screen.
 *
 * returns:
 *	Number of columns.
 */
extern int getcols();

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* int getmode()
 *	Return the current video mode number.
 *
 * returns:
 *	Video mode number.
 */
extern int getmode();

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* int getvstuff()
 *	Return video ?
 *
 * returns:
 *	Video ?
 */
extern int getvstuff();

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* gotorc(row, col)
 *	Position cursor on page 0.
 *
 * passed:
 *	int row := Row number.  Top of screen is zero.
 *	int col := Column number.  Left side of screen is zero.
 */
extern int gotorc(int row, int col);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* intswap(n)
 *	Swap bytes in 16-bit number.  1.2 becomes 2.1
 *
 * passed:
 *	int n := Number.
 *
 * returs:
 *	Number with bytes swapped.
 */
extern int intswap(int n);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* longswap(n)
 *	Swap bytes in 32-bit number.  1.2.3.4 becomes 4.3.2.1
 *
 * passed:
 *	long n := Number.
 *
 * returns:
 *	Number with bytes swapped.
 */
extern long longswap(long n);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* cll()
 *	Clear contents of current cursor row.
 */
extern int cll();

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* int pxe(api, param, seg)
 *	Run PXE API.
 *
 * passed:
 *	int api := PXE API number
 *	void *param := Pointer to parameter structure
 *	int seg := Segment of parameter structure (_DS() or _SS())
 *
 * returns:
 *	Exit code from PXE API.
 */
extern int pxe(int api, int *param, int seg);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* unsigned long get_memsize()
 *	Return amount of memory in system.
 *
 * returns:
 *	Amount of memory in Kbytes.
 */
unsigned long get_memsize();

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* int pmode_copy(to, from, words)
 *	Copy block of words using BIOS protected mode copy service.
 *
 * passed:
 *	unsigned long to := Copy to this address.
 *	unsigned long from := Copy from this address.
 *	unsigned words := Number of 16-bit words to copy.
 *
 * returns:
 *	Zero := Copy succeeded.
 *	Non-Zero := PC/AT BIOS failure code.
 */
int pmode_copy(unsigned long to, unsigned long from, unsigned words);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
int prepare_hardware(int *hdcount, char *mouse);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* unsigned long MK_FP(seg, off)
 *	Return far pointer from seg:off.
 *
 * passed:
 *	seg := 16-bit segment address
 *	off := 16-bit offset
 *
 * returns:
 *	(seg << 16) | off
 */
#define MK_FP(s, o) (((UINT32)(s) << 16) | (UINT16)(o))

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Macro version of intswap()
 */
#define INTSWAP(n) (((n) & 0xFF) << 8) | (((n) & 0xFF00) >> 8)

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Macro version of longswap()
 */
#define LONGSWAP(n)\
	(((UINT32)((n) & 0xFF) << 24) | ((UINT32)((n) & 0xFF00) << 8) |\
	(((UINT32)(n) & 0xFF0000L) >> 8) | (((UINT32)(n) & 0xFF000000L) >> 24))
