! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! This module must be updated when setup.s in the kernel loader is updated.
!

.global _SetupSeg
.global _kernel_addr
.global _kernel_size

.text

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Setup gets relocated to SETUPSEG:2000h.
!
Setup:
	push	cs
	pop	ds

	test	$11, #$01
	jnz	Setup_okay

!relocate zImage kernel here

Setup_okay:
	xor	bx, bx
	jmp	bx

gdt:	dw	0, 0, 0, 0	! dummy

	dw	0, 0, 0, 0	! unused

	dw	$FFFF		! 4G
	dw	0		! base addr = 0
	dw	$9A00		! code read/exec
	dw	$00CF		! gran=4K, 386

	dw	$FFFF		! 4G
	dw	0		! base addr = 0
	dw	$9200		! data read/write
	dw	$00CF		! gram=4K, 386

	dw	0
idt_48:	dw	0
	dw	0
	dw	0

gdt_48:	dw	$0800
gdt_48_addr_lo:
	dw	gdt - Setup
gdt_48_addr_hi:
	dw	0

EndSetup:

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
delay:
	jmp	delaydelay
delaydelay:
	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
empty_8042:
	push	ax

empty_8042_again:
	call	delay

	in	al,#$64
	test	al, #$01
	jz	empty_8042_test

	call	delay

	in	al, #$60
	jmp	empty_8042_again

empty_8042_test:
	test	al, #$02
	jnz	empty_8042_again

	pop	ax
	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! int Setup()
!	This 
!
! returns:
!	This can never return.
!
export _Setup
_Setup:
	!
	!
	!

	cli

	!
	! enable A20 gate
	!

	call	empty_8042

	mov	al, #$D1
	out	#$64, al

	call	empty_8042

	mov	al, #$DF
	out	#$60, al

	call	empty_8042

	!
	! Reset co-processor
	!

	xor	al, al
	out	#$F0, al

	call	delay

	out	#$F1, al

	call	delay

	!
	! Fixup gdt
	!

	mov	ax, _SetupSeg
	mov	dx, #16
	mul	dx

	add	ax, #$2000
	adc	dx, #0

	push	ds

	push	cs
	pop	ds

	add	gdt_48_addr_lo, ax
	adc	gdt_48_addr_hi, dx

	pop	ds

	!
	! Relocate the Setup procedure
	!

	push	ds

	mov	es, _SetupSeg
	mov	di, #$2000

	mov	si, cs
	mov	ds, si
	mov	si, #Setup

	mov	cx, #EndSetup
	sub	cx, #Setup
	inc	cx
	cld
	rep
	movsb

	pop	ds

	!
	! Reprogram 8258s
	!

	mov	al, #$80
	out	#$70, al

	mov	al, #$11
	out	#$20, al

	call	delay

	out	#$A0, al

	call	delay

	mov	al, #$20
	out	#$21, al

	call	delay

	mov	al, #$28
	out	#$A1, al

	call	delay

	mov	al, #$04
	out	#$21, al

	call	delay

	mov	al, #$02
	out	#$A1, al

	call	delay

	mov	al, #$01
	out	#$21, al

	call	delay

	out	#$A1, al

	call	delay

	mov	al, #$FF
	out	#$21, al

	call	delay

	mov	al, #$FB
	out	#$A1, al

	call	delay

	!
	! Code is relocated, interrupts are off, GDT is fixed up;
	! jump to the relocated Setup procedure and go, go, go ...
	!

	mov	ecx, _kernel_size

	mov	ax, _SetupSeg

	mov	sp, es
	mov	ss, sp
	mov	sp, #$4000
	sub	sp, #12

	push	ax
	push	#$2000

	lgdt	gdt_48

	retf

	hlt

!EOF
