/* undi_api.h */

#ifndef _UNDI_API_H
#define _UNDI_API_H

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*	Parameter structure and type definitions for TFTP API version 2.x
 *
 *	PXENV.H needs to be #included before this file.
 *
 *	All of the UNDI API services are still available after the stack
 *	has been unloaded.
 */


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* PXE UNDI API function numbers.
 */
 
#include "pxe_cmn.h"
 
#define PXENV_START_UNDI		0x0000
#define PXENV_UNDI_STARTUP	 	0x0001
#define PXENV_UNDI_CLEANUP	 	0x0002
#define PXENV_UNDI_INITIALIZE 		0x0003
#define PXENV_UNDI_RESET_NIC		0x0004
#define PXENV_UNDI_SHUTDOWN		0x0005
#define PXENV_UNDI_OPEN			0x0006
#define PXENV_UNDI_CLOSE		0x0007
#define PXENV_UNDI_TRANSMIT		0x0008
#define PXENV_UNDI_SET_MCAST_ADDR	0x0009
#define PXENV_UNDI_SET_STATION_ADDR	0x000A
#define PXENV_UNDI_SET_PACKET_FILTER	0x000B
#define PXENV_UNDI_GET_INFORMATION	0x000C
#define PXENV_UNDI_GET_STATISTICS	0x000D
#define PXENV_UNDI_CLEAR_STATISTICS	0x000E
#define PXENV_UNDI_INITIATE_DIAGS	0x000F
#define PXENV_UNDI_FORCE_INTERRUPT	0x0010
#define PXENV_UNDI_GET_MCAST_ADDR	0x0011
#define PXENV_UNDI_GET_NIC_TYPE		0x0012
#define PXENV_UNDI_GET_NDIS_INFO	0x0013
#define PXENV_UNDI_ISR			0x0014
#define PXENV_STOP_UNDI			0x0015
#define PXENV_UNDI_GET_STATE		0x0016


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#define ADDR_LEN			16
#define MAXNUM_MCADDR			8


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNDI_MCAST_ADDR {
	UINT16 MCastAddrCount;		/* In: Number of multi-cast */
					/* addresses. */
	UINT8 MCastAddr[MAXNUM_MCADDR][ADDR_LEN]; /* In: */
					/* list of multi-cast addresses. */
					/* Each address can take up to */
					/* ADDR_LEN bytes and a maximum */
					/* of MAXNUM_MCADDR address can */
					/* be provided*/
} t_PXENV_UNDI_MCAST_ADDR;


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Definitions of TFTP API parameter structures.
 */

typedef struct s_PXENV_START_UNDI {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
	UINT16 ax;			/* In: These register fields must be */
	UINT16 bx;			/*     filled in with the same data */
	UINT16 dx;			/*     that was passed to the MLID */
	UINT16 di;			/*     option ROM boot code by the */
	UINT16 es;			/*     system BIOS. */
} t_PXENV_START_UNDI;


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNDI_STARTUP {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
} t_PXENV_UNDI_STARTUP;


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNDI_CLEANUP {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
} t_PXENV_UNDI_CLEANUP;


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNDI_INITIALIZE {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
	UINT32 ProtocolIni;		/* In: See description below */
	UINT8 reserved[8];
} t_PXENV_UNDI_INITIALIZE;

/*
	ProtocolIni :  

    This is an input parameter and is a 32-bit physical address of
    a memory  copy of the  driver module in  the protocol.ini file
    obtained from the  Protocol Manager  driver(refer to  NDIS 2.0
    specifications).   This parameter  is basically  supported for
    the universal NDIS driver to pass the information contained in
    protocol.ini   file  to  the  NIC   driver  for  any  specific
    configuration of   the   NIC.      (Note   that   the   module
    identification in the  protocol.ini  file  was  done  by  NDIS
    itself.)  This value can be NULL for for any other application
    interfacing to the Universal NIC Driver.
*/


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNDI_RESET {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
	t_PXENV_UNDI_MCAST_ADDR R_Mcast_Buf; /* multicast address list */
					/* see note below  */
} t_PXENV_UNDI_RESET;

  /* Note: The  NIC  driver  does  not  remember  the  multicast
    addresses provided in any  call.    So  the  application  must
    provide the multicast address  list with all  the calls that
    reset the receive unit of the adapter.  
  */


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNDI_SHUTDOWN {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
} t_PXENV_UNDI_SHUTDOWN;


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNDI_OPEN {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
	UINT16 OpenFlag;		/* In: See description below */
	UINT16 PktFilter;		/* In: Filter for receiving */
					/* packet. It takes the following */
					/* values, multiple values can be */
					/* ORed together. */

#define FLTR_DIRECTED	0x0001  	/* directed/multicast */
#define FLTR_BRDCST	0x0002  	/* broadcast packets */
#define FLTR_PRMSCS	0x0004  	/* any packet on LAN */
#define FLTR_SRC_RTG	0x0008  	/* source routing packet */

	t_PXENV_UNDI_MCAST_ADDR McastBuffer; /* In: */
					/* See t_PXENV_UNDI_MCAST_ADDR. */
} t_PXENV_UNDI_OPEN;

/*  OpenFlag:  
    This is  an input parameter and is  adapter specific.  This is
    supported  for Universal NDIS 2.0 driver to pass down the Open
    flags  provided  by   the  protocol   driver  (See   NDIS  2.0
    specifications).  This can be zero.  
*/


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNDI_CLOSE {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
} t_PXENV_UNDI_CLOSE;


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* TBD - Transmit Block Descriptor
 */

#define	MAX_DATA_BLKS		8

typedef	struct s_PXENV_UNDI_TBD {
	UINT16 ImmedLength;		/* In: Data buffer length in */
					/* bytes. */
	UINT16 XmitOffset;		/* 16-bit segment & offset of the */
	UINT16 XmitSegment;		/* immediate data buffer. */
	UINT16 DataBlkCount;		/* In: Number of data blocks. */
	struct DataBlk {
		UINT8 TDPtrType;	/* 0 => 32 bit Phys pointer in */
					/* TDDataPtr, not supported in this */
					/* version of LSA */
					/* 1 => seg:offser in TDDataPtr which */
					/* can be a real mode or 16-bit  */
					/* protected mode pointer */
		UINT8 TDRsvdByte;	/* Reserved, must be zero. */
		UINT16 TDDataLen;	/* Data block length in bytes. */
		UINT32 TDDataPtr;	/* Far pointer to data buffer. */
	} DataBlock[MAX_DATA_BLKS];
} t_PXENV_UNDI_TBD;


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNDI_TRANSMIT {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
	UINT8	Protocol;		/* See description below */
#define P_UNKNOWN	0
#define P_IP	 	1
#define P_ARP		2
#define P_RARP		3

	UINT8	XmitFlag;		/* See description below	*/
#define XMT_DESTADDR	0x0000		/* destination address given */
#define XMT_BROADCAST	0x0001		/* use broadcast address */

	UINT16 DestAddrOffset;		/* 16-bit segment & offset of the */
	UINT16 DestAddrSegment;		/* destination media address */
					/*  See description below */
	UINT16 TBDOffset;		/* 16-bit segment & offset of the */
	UINT16 TBDSegment;		/* transmit buffer descriptor of type */
					/* XmitBufferDesc	*/
	UINT32 Reserved[2];		/* for future use	*/
} t_PXENV_UNDI_TRANSMIT;

/* 
    Protocol:

    This is the protocol  of  the  upper  layer  that  is  calling
    NICTransmit call.   If the  upper layer  has filled  the media
    header this field must be 0.

    XmitFlag:

    If  this flag is  0, the NIC  driver expects a  pointer to the
    destination media  address in the field  DestMediaAddr.  If 1,
    the   NIC  driver   fills  the   broadcast  address   for  the
    destination.
	
    DestAddrOffset & DestAddrSegment:

    This  is a pointer to the  hardware address of the destination
    media.  It  can be  null if  the destination  is not  known in
    which case the XmitFlag contains 1 for broadcast.  Destination
    media address  must be  obtained by  the upper  level protocol
    (with  Address Resolution Protocol) and NIC driver does not do
    any address resolution.
*/


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNDI_SET_MCAST_ADDR {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
	t_PXENV_UNDI_MCAST_ADDR McastBuffer; /* In: */
					/* See t_PXENV_UNDI_MCAST_ADDR. */
} t_PXENV_UNDI_SET_MCAST_ADDR;


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNDI_SET_STATION_ADDR {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
	UINT8  StationAddress[ADDR_LEN]; /* new address to be set */
} t_PXENV_UNDI_SET_STATION_ADDR;


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNDI_SET_PACKET_FILTER {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
	UINT8 filter;			/* In: Receive filter value. */
					/* see t_PXENV_UNDI_OPEN for values */
} t_PXENV_UNDI_SET_PACKET_FILTER;	


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNDI_GET_INFORMATION {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
	UINT16 BaseIo;			/* Out: Adapter's Base IO */
	UINT16 IntNumber;		/* Out: IRQ number */
	UINT16 MaxTranUnit;		/* Out: MTU	*/
	UINT16  HwType;			/* Out: type of protocol at hardware level */

#define ETHER_TYPE	1
#define EXP_ETHER_TYPE	2
#define IEEE_TYPE	6
#define ARCNET_TYPE	7

    /*  other numbers can  be obtained from  rfc1010 for "Assigned
    Numbers".  This number may not be validated by the application
    and hence adding new numbers to the list should be fine at any
    time.  */

	UINT16 HwAddrLen;		/* Out: actual length of hardware address */
	UINT8 CurrentNodeAddress[ADDR_LEN]; /* Out: Current hardware address*/
	UINT8 PermNodeAddress[ADDR_LEN]; /* Out: Permanent hardware address*/
	UINT16 ROMAddress;		/* Out: ROM address */
	UINT16 RxBufCt;			/* Out: receive Queue length	*/
	UINT16 TxBufCt;			/* Out: Transmit Queue length */
} t_PXENV_UNDI_GET_INFORMATION;


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNDI_GET_STATISTICS {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
	UINT32 XmtGoodFrames; 		/* Out: No. of good transmissions */
	UINT32 RcvGoodFrames;		/* Out: No. of good frames received */
	UINT32 RcvCRCErrors;   		/* Out: No. of frames with CRC error */
	UINT32 RcvResourceErrors;  	/* Out: no. of frames discarded - */ 
					/* Out: receive Queue full */
} t_PXENV_UNDI_GET_STATISTICS;


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNDI_CLEAR_STATISTICS {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
} t_PXENV_UNDI_CLEAR_STATISTICS;


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNDI_INITIATE_DIAGS {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
} t_PXENV_UNDI_INITIATE_DIAGS;


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNDI_FORCE_INTERRUPT {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
} t_PXENV_UNDI_FORCE_INTERRUPT;


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNDI_GET_MCAST_ADDR {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
	UINT32 InetAddr;		/* In: IP Multicast Address */
	UINT8  MediaAddr[ADDR_LEN];	/* Out: corresponding hardware */
					/*      multicast address */
} t_PXENV_UNDI_GET_MCAST_ADDR;


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNDI_GET_NIC_TYPE {
	UINT16 Status;			/* OUT: PXENV_STATUS_xxx */
	UINT8 NicType;			/* OUT: 2=PCI, 3=PnP */
	union {
		struct {
		   	UINT16 Vendor_ID; /* OUT:  */
			UINT16 Dev_ID;  /* OUT:  */
			UINT8 Base_Class; /* OUT: */
			UINT8 Sub_Class;  /* OUT: */
			UINT8 Prog_Intf;  /* OUT: program interface */ 
			UINT8 Rev;        /* OUT: Revision number */ 
			UINT16 BusDevFunc; /* OUT: Bus, Device */        
						/* & Function numbers */
			UINT16 SubVendor_ID;	/* OUT: */
			UINT16 SubDevice_ID;	/* OUT: */
		} pci;
		struct {
			UINT32 EISA_Dev_ID; /* Out:  */
			UINT8 Base_Class; /* OUT: */
			UINT8 Sub_Class;  /* OUT: */
			UINT8 Prog_Intf;  /* OUT: program interface */
 			UINT16 CardSelNum; /* OUT: Card Selector Number */
			UINT8 Reserved;	/* to make it 10 bytes */
		} pnp;
	} pci_pnp_info;
} t_PXENV_UNDI_GET_NIC_TYPE;

	
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNDI_GET_NDIS_INFO {
	UINT16 Status;			/* OUT: PXENV_STATUS_xxx */
	UINT8 IfaceType[16];		/* OUT: Type name of MAC, AsciiZ */
					/*      format.  This is used by the */
					/*      Universal NDIS Driver to fill */
					/*      the driver type in it's MAC */
					/*      Service specific */
					/*      characteristic table */
	UINT32 LinkSpeed;		/* OUT:	*/ 
	UINT32 ServiceFlags;		/* OUT: as defined in NDIS Spec 2.0X */
	UINT32 Reserved[4];		/* OUT: will be filled with 0s till */
					/*      defined */
} t_PXENV_UNDI_GET_NDIS_INFO;


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNDI_ISR {
	UINT16 Status;			/* OUT: PXENV_STATUS_xxx */
	UINT16 FuncFlag;		/* In: PXENV_UNDI_ISR_IN_xxx */
					/* Out: PXENV_UNDI_ISR_OUT_xxx */
	UINT16 BufferLength;		/* Out: %% */
	UINT16 FrameLength;		/* Out: %% */
	UINT16 FrameHeaderLength;	/* Out: %% */
	UINT16 FrameOffset;		/* Out: %% */
	UINT16 FrameSegSel;		/* Out: %% */
	UINT8  ProtType;		/* OUT: %% */
	UINT8  PktType;			/* OUT: %% */
} t_PXENV_UNDI_ISR;


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#define PXENV_UNDI_ISR_IN_START		1 /* This function must be first */
					  /* when an interrupt is received. */
					  /* It will tell us if the intr */
					  /* was generated by our device. */

#define PXENV_UNDI_ISR_IN_PROCESS	2 /* Call to start processing one of */
					  /* our interrupts. */

#define PXENV_UNDI_ISR_IN_GET_NEXT	3 /* Call to start/continue receiving */
					  /* data from receive buffer(s). */


/* Possible responses from PXENV_UNDI_ISR_IN_START
 */
#define PXENV_UNDI_ISR_OUT_OURS		0 /* This is our interrupt.  Deal */
					  /* with it. */

#define PXENV_UNDI_USR_OUT_NOT_OURS	1 /* This is not our interrupt. */
					  /* Ignore it. */


/* Possible responses from PXENV_UNDI_ISR_IN_PROCESS and
 * PXENV_UNDI_ISR_IN_PROCESS
 */
#define PXENV_UNDI_ISR_OUT_DONE		0 /* We are done processing this */
					  /* interrupt. */

#define PXENV_UNDI_ISR_OUT_TRANSMIT	2 /* We completed a transmit */
					  /* interrupt. */

#define PXENV_UNDI_ISR_OUT_RECEIVE	3 /* Get data from receive buffer. */

#define PXENV_UNDI_ISR_OUT_BUSY		4 /* ? */


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_STOP_UNDI {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
} t_PXENV_STOP_UNDI;


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#define PXENV_UNDI_STARTED			1 /* not even initialized */
#define PXENV_UNDI_INITIALIZED		2 /* initialized and closed (not opened) */
#define PXENV_UNDI_OPENED  			3 /* initialized & opened */

typedef struct s_PXENV_UNDI_GET_STATE {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
	UINT16 UNDI_State;
} t_PXENV_UNDI_GET_STATE;




#endif /* _UNDI_API_H */
